@@ -1,8 +1,6 @@
 # --
 # Kernel/Modules/AgentTicketBulk.pm - to do bulk actions on tickets
-# Copyright (C) 2001-2010 OTRS AG, http://otrs.org/
-# --
-# $Id: AgentTicketBulk.pm,v 1.69 2010/11/02 13:42:43 mg Exp $
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -17,9 +15,9 @@ use warnings;
 use Kernel::System::State;
 use Kernel::System::Priority;
 use Kernel::System::LinkObject;
-
-use vars qw($VERSION);
-$VERSION = qw($Revision: 1.69 $) [1];
+use Kernel::System::Web::UploadCache;
+use Kernel::System::CustomerUser;
+use Kernel::System::TemplateGenerator;
 
 sub new {
     my ( $Type, %Param ) = @_;
@@ -35,19 +33,95 @@ sub new {
         }
     }
 
-    $Self->{StateObject}     = Kernel::System::State->new(%Param);
-    $Self->{PriorityObject}  = Kernel::System::Priority->new(%Param);
-    $Self->{LinkObject}      = Kernel::System::LinkObject->new(%Param);
-    $Self->{CheckItemObject} = Kernel::System::CheckItem->new(%Param);
-
+    $Self->{StateObject}             = Kernel::System::State->new(%Param);
+    $Self->{PriorityObject}          = Kernel::System::Priority->new(%Param);
+    $Self->{LinkObject}              = Kernel::System::LinkObject->new(%Param);
+    $Self->{CheckItemObject}         = Kernel::System::CheckItem->new(%Param);
+    $Self->{UploadCacheObject}       = Kernel::System::Web::UploadCache->new(%Param);
+    $Self->{CustomerUserObject}      = Kernel::System::CustomerUser->new(%Param);
+    $Self->{TemplateGeneratorObject} = Kernel::System::TemplateGenerator->new(
+        %Param,
+        CustomerUserObject => $Self->{CustomerUserObject},
+    );
     $Self->{Config} = $Self->{ConfigObject}->Get("Ticket::Frontend::$Self->{Action}");
 
+    # get form id for note
+    $Self->{NoteFormID} = $Self->{ParamObject}->GetParam( Param => 'NoteFormID' );
+
+    # create form id for note
+    if ( !$Self->{NoteFormID} ) {
+        $Self->{NoteFormID} = $Self->{UploadCacheObject}->FormIDCreate();
+    }
+
+    # get form id for email
+    $Self->{EmailFormID} = $Self->{ParamObject}->GetParam( Param => 'EmailFormID' );
+
+    # create form id for email
+    if ( !$Self->{EmailFormID} ) {
+        $Self->{EmailFormID} = $Self->{UploadCacheObject}->FormIDCreate();
+    }
+
     return $Self;
 }
 
 sub Run {
     my ( $Self, %Param ) = @_;
 
+    if ( $Self->{Subaction} eq 'CancelAndUnlockTickets' ) {
+
+        my @TicketIDs
+            = grep {$_}
+            $Self->{ParamObject}->GetArray( Param => 'LockedTicketID' );
+
+        # challenge token check for write action
+        $Self->{LayoutObject}->ChallengeTokenCheck();
+
+        # check needed stuff
+        if ( !@TicketIDs ) {
+            return $Self->{LayoutObject}->ErrorScreen(
+                Message => 'Can\'t lock Tickets, no TicketIDs are given!',
+                Comment => 'Please contact the admin.',
+            );
+        }
+
+        my $Message = '';
+
+        TICKET_ID:
+        for my $TicketID (@TicketIDs) {
+
+            my $Access = $Self->{TicketObject}->TicketPermission(
+                Type     => 'lock',
+                TicketID => $TicketID,
+                UserID   => $Self->{UserID}
+            );
+
+            # error screen, don't show ticket
+            if ( !$Access ) {
+                return $Self->{LayoutObject}->NoPermission( WithHeader => 'yes' );
+            }
+
+            # set unlock
+            my $Lock = $Self->{TicketObject}->TicketLockSet(
+                TicketID => $TicketID,
+                Lock     => 'unlock',
+                UserID   => $Self->{UserID},
+            );
+            if ( !$Lock ) {
+                $Message .= "$TicketID,";
+            }
+        }
+
+        if ( $Message ne '' ) {
+            return $Self->{LayoutObject}
+                ->ErrorScreen( Message => "Ticket ($Message) is not unlocked!", );
+        }
+
+        return $Self->{LayoutObject}->Redirect(
+            OP => $Self->{LastScreenOverview},
+        );
+
+    }
+
     # check if bulk feature is enabled
     if ( !$Self->{ConfigObject}->Get('Ticket::Frontend::BulkFeature') ) {
         return $Self->{LayoutObject}->ErrorScreen(
@@ -55,7 +129,7 @@ sub Run {
         );
     }
 
-    # get involved tickets, filterung empty TicketIDs
+    # get involved tickets, filtering empty TicketIDs
     my @TicketIDs
         = grep {$_}
         $Self->{ParamObject}->GetArray( Param => 'TicketID' );
@@ -64,7 +138,7 @@ sub Run {
     if ( !@TicketIDs ) {
         return $Self->{LayoutObject}->ErrorScreen(
             Message => 'No TicketID is given!',
-            Comment => 'You need minimum one selected ticket!',
+            Comment => 'You need at least one selected ticket!',
         );
     }
     my $Output .= $Self->{LayoutObject}->Header(
@@ -79,16 +153,24 @@ sub Run {
     # get all parameters and check for errors
     if ( $Self->{Subaction} eq 'Do' ) {
 
+        # challenge token check for write action
+        $Self->{LayoutObject}->ChallengeTokenCheck();
+
         # get all parameters
         for my $Key (
             qw(OwnerID Owner ResponsibleID Responsible PriorityID Priority QueueID Queue Subject
-            Body ArticleTypeID ArticleType StateID State MergeToSelection MergeTo LinkTogether
-            LinkTogetherParent Unlock MergeToChecked MergeToOldestChecked TimeUnits)
+            Body ArticleTypeID ArticleType TypeID StateID State MergeToSelection MergeTo LinkTogether
+            EmailSubject EmailBody EmailTimeUnits
+            LinkTogetherParent Unlock MergeToChecked MergeToOldestChecked)
             )
         {
             $GetParam{$Key} = $Self->{ParamObject}->GetParam( Param => $Key ) || '';
         }
 
+        for my $Key (qw(TimeUnits)) {
+            $GetParam{$Key} = $Self->{ParamObject}->GetParam( Param => $Key );
+        }
+
         # get time stamp based on user time zone
         %Time = $Self->{LayoutObject}->TransfromDateSelection(
             Year   => $Self->{ParamObject}->GetParam( Param => 'Year' ),
@@ -107,27 +189,68 @@ sub Run {
 
         # check some stuff
         if (
-            ( $Self->{ConfigObject}->Get('Ticket::Frontend::NeedAccountedTime') )
-            && !defined $GetParam{TimeUnits}
+            $GetParam{Subject}
+            &&
+            $Self->{ConfigObject}->Get('Ticket::Frontend::AccountTime')
+            && $Self->{ConfigObject}->Get('Ticket::Frontend::NeedAccountedTime')
+            && $GetParam{TimeUnits} eq ''
             )
         {
             $Error{'TimeUnitsInvalid'} = 'ServerError';
         }
-        if ( !$GetParam{'Subject'} ) {
-            $Error{'SubjectInvalid'} = 'ServerError';
-        }
-        if ( !$GetParam{'Body'} ) {
-            $Error{'RichTextInvalid'} = 'ServerError';
-        }
-        if ( !$Self->{TimeObject}->Date2SystemTime( %Time, Second => 0 ) ) {
-            $Error{'DateInvalid'} = 'ServerError';
-        }
+
         if (
-            $Self->{TimeObject}->Date2SystemTime( %Time, Second => 0 )
-            < $Self->{TimeObject}->SystemTime()
+            $GetParam{EmailSubject}
+            &&
+            $Self->{ConfigObject}->Get('Ticket::Frontend::AccountTime')
+            && $Self->{ConfigObject}->Get('Ticket::Frontend::NeedAccountedTime')
+            && $GetParam{EmailTimeUnits} eq ''
             )
         {
-            $Error{'DateInvalid'} = 'ServerError';
+            $Error{'EmailTimeUnitsInvalid'} = 'ServerError';
+        }
+
+        # Body and Subject must both be filled in or both be empty
+        if ( $GetParam{Subject} eq '' && $GetParam{Body} ne '' ) {
+            $Error{'SubjectInvalid'} = 'ServerError';
+        }
+        if ( $GetParam{Subject} ne '' && $GetParam{Body} eq '' ) {
+            $Error{'BodyInvalid'} = 'ServerError';
+        }
+
+        # Email Body and Email Subject must both be filled in or both be empty
+        if ( $GetParam{EmailSubject} eq '' && $GetParam{EmailBody} ne '' ) {
+            $Error{'EmailSubjectInvalid'} = 'ServerError';
+        }
+        if ( $GetParam{EmailSubject} ne '' && $GetParam{EmailBody} eq '' ) {
+            $Error{'EmailBodyInvalid'} = 'ServerError';
+        }
+
+        # check if pending date must be validated
+        if ( $GetParam{StateID} || $GetParam{State} ) {
+            my %StateData;
+            if ( $GetParam{StateID} ) {
+                %StateData = $Self->{TicketObject}->{StateObject}->StateGet(
+                    ID => $GetParam{StateID},
+                );
+            }
+            else {
+                %StateData = $Self->{TicketObject}->{StateObject}->StateGet(
+                    Name => $GetParam{State},
+                );
+            }
+            if ( $StateData{TypeName} =~ /^pending/i ) {
+                if ( !$Self->{TimeObject}->Date2SystemTime( %Time, Second => 0 ) ) {
+                    $Error{'DateInvalid'} = 'ServerError';
+                }
+                if (
+                    $Self->{TimeObject}->Date2SystemTime( %Time, Second => 0 )
+                    < $Self->{TimeObject}->SystemTime()
+                    )
+                {
+                    $Error{'DateInvalid'} = 'ServerError';
+                }
+            }
         }
         if ( $GetParam{'MergeToSelection'} eq 'OptionMergeTo' && $GetParam{'MergeTo'} ) {
             $Self->{CheckItemObject}->StringClean(
@@ -159,10 +282,17 @@ sub Run {
 
     # process tickets
     my @TicketIDSelected;
-    my $ActionFlag = 0;
+    my $LockedTickets = '';
+    my $ActionFlag    = 0;
+    my $Counter       = 1;
+    $Param{TicketsWereLocked} = 0;
+
     TICKET_ID:
     for my $TicketID (@TicketIDs) {
-        my %Ticket = $Self->{TicketObject}->TicketGet( TicketID => $TicketID );
+        my %Ticket = $Self->{TicketObject}->TicketGet(
+            TicketID      => $TicketID,
+            DynamicFields => 0,
+        );
 
         # check permissions
         my $Access = $Self->{TicketObject}->TicketPermission(
@@ -174,17 +304,16 @@ sub Run {
 
             # error screen, don't show ticket
             $Output .= $Self->{LayoutObject}->Notify(
-                Data => $Ticket{TicketNumber} . ': $Text{"No access to ticket!"}',
+                Data => $Ticket{TicketNumber}
+                    . ': $Text{"You don\'t have write access to this ticket."}',
             );
             next TICKET_ID;
         }
 
-        $Param{TicketsWereLocked} = 0;
-
         # check if it's already locked by somebody else
         if ( !$Self->{Config}->{RequiredLock} ) {
             $Output .= $Self->{LayoutObject}->Notify(
-                Data => $Ticket{TicketNumber} . ': $Text{"Ticket is used!"}',
+                Data => $Ticket{TicketNumber} . ': $Text{"Ticket selected."}',
             );
         }
         else {
@@ -196,13 +325,14 @@ sub Run {
                 if ( !$AccessOk ) {
                     $Output .= $Self->{LayoutObject}->Notify(
                         Data => $Ticket{TicketNumber}
-                            . ': $Text{"Ticket is locked for another agent!"}',
+                            . ': $Text{"Ticket is locked by another agent."}',
                     );
                     next TICKET_ID;
                 }
             }
             else {
                 $Param{TicketsWereLocked} = 1;
+                $LockedTickets .= "LockedTicketID=" . $TicketID . ';';
             }
 
             # set lock
@@ -219,7 +349,7 @@ sub Run {
                 NewUserID => $Self->{UserID},
             );
             $Output .= $Self->{LayoutObject}->Notify(
-                Data => $Ticket{TicketNumber} . ': $Text{"Ticket locked!"}',
+                Data => $Ticket{TicketNumber} . ': $Text{"Ticket locked."}',
             );
         }
 
@@ -244,7 +374,8 @@ sub Run {
 
             # set responsible
             if (
-                $Self->{Config}->{Responsible}
+                $Self->{ConfigObject}->Get('Ticket::Responsible')
+                && $Self->{Config}->{Responsible}
                 && ( $GetParam{'ResponsibleID'} || $GetParam{'Responsible'} )
                 )
             {
@@ -270,6 +401,17 @@ sub Run {
                 );
             }
 
+            # set type
+            if ( $Self->{ConfigObject}->Get('Ticket::Type') && $Self->{Config}->{TicketType} ) {
+                if ( $GetParam{'TypeID'} ) {
+                    $Self->{TicketObject}->TicketTypeSet(
+                        TypeID   => $GetParam{'TypeID'},
+                        TicketID => $TicketID,
+                        UserID   => $Self->{UserID},
+                    );
+                }
+            }
+
             # set queue
             if ( $GetParam{'QueueID'} || $GetParam{'Queue'} ) {
                 $Self->{TicketObject}->TicketQueueSet(
@@ -280,6 +422,84 @@ sub Run {
                 );
             }
 
+            # send email
+            my $EmailArticleID;
+            if (
+                $GetParam{'EmailSubject'}
+                && $GetParam{'EmailBody'}
+                )
+            {
+                my $MimeType = 'text/plain';
+                if ( $Self->{LayoutObject}->{BrowserRichText} ) {
+                    $MimeType = 'text/html';
+
+                    # verify html document
+                    $GetParam{'EmailBody'} = $Self->{LayoutObject}->RichTextDocumentComplete(
+                        String => $GetParam{'EmailBody'},
+                    );
+                }
+
+                # get customer email address
+                my $Customer;
+                if ( $Ticket{CustomerUserID} ) {
+                    my %Customer = $Self->{CustomerUserObject}->CustomerUserDataGet(
+                        User => $Ticket{CustomerUserID}
+                    );
+                    if ( $Customer{UserEmail} ) {
+                        $Customer = $Customer{UserEmail};
+                    }
+                }
+
+                # check if we have an address, otherwise deduct it from the articles
+                if ( !$Customer ) {
+                    my %Data = $Self->{TicketObject}->ArticleLastCustomerArticle(
+                        TicketID      => $TicketID,
+                        DynamicFields => 0,
+                    );
+
+                    # use ReplyTo if set, otherwise use From
+                    $Customer = $Data{ReplyTo} ? $Data{ReplyTo} : $Data{From};
+
+                    # check article type and replace To with From (in case)
+                    if ( $Data{SenderType} !~ /customer/ ) {
+
+                        # replace From/To, To/From because sender is agent
+                        $Customer = $Data{To};
+                    }
+
+                }
+
+                # generate sender name
+                my $From = $Self->{TemplateGeneratorObject}->Sender(
+                    QueueID => $Ticket{QueueID},
+                    UserID  => $Self->{UserID},
+                );
+
+                # generate subject
+                my $TicketNumber
+                    = $Self->{TicketObject}->TicketNumberLookup( TicketID => $TicketID );
+
+                my $EmailSubject = $Self->{TicketObject}->TicketSubjectBuild(
+                    TicketNumber => $TicketNumber,
+                    Subject => $GetParam{EmailSubject} || '',
+                );
+
+                $EmailArticleID = $Self->{TicketObject}->ArticleSend(
+                    TicketID       => $TicketID,
+                    ArticleType    => 'email-external',
+                    SenderType     => 'agent',
+                    From           => $From,
+                    To             => $Customer,
+                    Subject        => $EmailSubject,
+                    Body           => $GetParam{EmailBody},
+                    MimeType       => $MimeType,
+                    Charset        => $Self->{LayoutObject}->{UserCharset},
+                    UserID         => $Self->{UserID},
+                    HistoryType    => 'SendAnswer',
+                    HistoryComment => '%%' . $Customer,
+                );
+            }
+
             # add note
             my $ArticleID;
             if (
@@ -321,7 +541,10 @@ sub Run {
                     State    => $GetParam{'State'},
                     UserID   => $Self->{UserID},
                 );
-                my %Ticket = $Self->{TicketObject}->TicketGet( TicketID => $TicketID );
+                my %Ticket = $Self->{TicketObject}->TicketGet(
+                    TicketID      => $TicketID,
+                    DynamicFields => 0,
+                );
                 my %StateData = $Self->{TicketObject}->{StateObject}->StateGet(
                     ID => $Ticket{StateID},
                 );
@@ -347,14 +570,52 @@ sub Run {
                 }
             }
 
-            # time units
-            if ( $GetParam{'TimeUnits'} ) {
-                $Self->{TicketObject}->TicketAccountTime(
-                    TicketID  => $TicketID,
-                    ArticleID => $ArticleID,
-                    TimeUnit  => $GetParam{'TimeUnits'},
-                    UserID    => $Self->{UserID},
-                );
+            # time units for note
+            if ( $GetParam{'TimeUnits'} && $ArticleID ) {
+                if ( $Self->{ConfigObject}->Get('Ticket::Frontend::BulkAccountedTime') ) {
+                    $Self->{TicketObject}->TicketAccountTime(
+                        TicketID  => $TicketID,
+                        ArticleID => $ArticleID,
+                        TimeUnit  => $GetParam{'TimeUnits'},
+                        UserID    => $Self->{UserID},
+                    );
+                }
+                elsif (
+                    !$Self->{ConfigObject}->Get('Ticket::Frontend::BulkAccountedTime')
+                    && $Counter == 1
+                    )
+                {
+                    $Self->{TicketObject}->TicketAccountTime(
+                        TicketID  => $TicketID,
+                        ArticleID => $ArticleID,
+                        TimeUnit  => $GetParam{'TimeUnits'},
+                        UserID    => $Self->{UserID},
+                    );
+                }
+            }
+
+            # time units for email
+            if ( $GetParam{ 'EmailTimeUnits' && $EmailArticleID } ) {
+                if ( $Self->{ConfigObject}->Get('Ticket::Frontend::BulkAccountedTime') ) {
+                    $Self->{TicketObject}->TicketAccountTime(
+                        TicketID  => $TicketID,
+                        ArticleID => $EmailArticleID,
+                        TimeUnit  => $GetParam{'EmailTimeUnits'},
+                        UserID    => $Self->{UserID},
+                    );
+                }
+                elsif (
+                    !$Self->{ConfigObject}->Get('Ticket::Frontend::BulkAccountedTime')
+                    && $Counter == 1
+                    )
+                {
+                    $Self->{TicketObject}->TicketAccountTime(
+                        TicketID  => $TicketID,
+                        ArticleID => $EmailArticleID,
+                        TimeUnit  => $GetParam{'EmailTimeUnits'},
+                        UserID    => $Self->{UserID},
+                    );
+                }
             }
 
             # merge to
@@ -378,7 +639,10 @@ sub Run {
                 my $TicketIDOldest;
                 my $TicketIDOldestID;
                 for my $TicketIDCheck (@TicketIDs) {
-                    my %Ticket = $Self->{TicketObject}->TicketGet( TicketID => $TicketIDCheck );
+                    my %Ticket = $Self->{TicketObject}->TicketGet(
+                        TicketID      => $TicketIDCheck,
+                        DynamicFields => 0,
+                    );
                     if ( !defined $TicketIDOldest ) {
                         $TicketIDOldest   = $Ticket{CreateTimeUnix};
                         $TicketIDOldestID = $TicketIDCheck;
@@ -399,16 +663,13 @@ sub Run {
                 }
             }
 
-            # link with
+            # link all tickets to a parent
+            if ( $GetParam{'LinkTogetherParent'} ) {
+                my $MainTicketID = $Self->{TicketObject}->TicketIDLookup(
+                    TicketNumber => $GetParam{'LinkTogetherParent'},
+                );
 
-            # link togehter
-            if ( $GetParam{'LinkTogether'} ) {
-
-                # link parent
-                if ( $GetParam{'LinkTogetherParent'} ) {
-                    my $MainTicketID = $Self->{TicketObject}->TicketIDLookup(
-                        TicketNumber => $GetParam{'LinkTogetherParent'},
-                    );
+                for my $TicketIDPartner (@TicketIDs) {
                     if ( $MainTicketID ne $TicketID ) {
                         $Self->{LinkObject}->LinkAdd(
                             SourceObject => 'Ticket',
@@ -421,21 +682,21 @@ sub Run {
                         );
                     }
                 }
+            }
 
-                # link normal
-                else {
-                    for my $TicketIDPartner (@TicketIDs) {
-                        if ( $TicketID ne $TicketIDPartner ) {
-                            $Self->{LinkObject}->LinkAdd(
-                                SourceObject => 'Ticket',
-                                SourceKey    => $TicketID,
-                                TargetObject => 'Ticket',
-                                TargetKey    => $TicketIDPartner,
-                                Type         => 'Normal',
-                                State        => 'Valid',
-                                UserID       => $Self->{UserID},
-                            );
-                        }
+            # link together
+            if ( $GetParam{'LinkTogether'} ) {
+                for my $TicketIDPartner (@TicketIDs) {
+                    if ( $TicketID ne $TicketIDPartner ) {
+                        $Self->{LinkObject}->LinkAdd(
+                            SourceObject => 'Ticket',
+                            SourceKey    => $TicketID,
+                            TargetObject => 'Ticket',
+                            TargetKey    => $TicketIDPartner,
+                            Type         => 'Normal',
+                            State        => 'Valid',
+                            UserID       => $Self->{UserID},
+                        );
                     }
                 }
             }
@@ -450,6 +711,7 @@ sub Run {
             }
             $ActionFlag = 1;
         }
+        $Counter++;
     }
 
     # redirect
@@ -463,8 +725,9 @@ sub Run {
         %Param,
         %GetParam,
         %Time,
-        TicketIDs => \@TicketIDSelected,
-        Errors    => \%Error,
+        TicketIDs     => \@TicketIDSelected,
+        LockedTickets => $LockedTickets,
+        Errors        => \%Error,
     );
     $Output .= $Self->{LayoutObject}->Footer(
         Type => 'Small',
@@ -477,7 +740,7 @@ sub _Mask {
 
     # prepare errors!
     if ( $Param{Errors} ) {
-        for my $KeyError ( keys %{ $Param{Errors} } ) {
+        for my $KeyError ( sort keys %{ $Param{Errors} } ) {
             $Param{$KeyError}
                 = $Self->{LayoutObject}->Ascii2Html( Text => $Param{Errors}->{$KeyError} );
         }
@@ -503,16 +766,26 @@ sub _Mask {
     # build ArticleTypeID string
     my %DefaultNoteTypes = %{ $Self->{Config}->{ArticleTypes} };
     my %NoteTypes = $Self->{TicketObject}->ArticleTypeList( Result => 'HASH' );
-    for my $KeyNoteType ( keys %NoteTypes ) {
+    for my $KeyNoteType ( sort keys %NoteTypes ) {
         if ( !$DefaultNoteTypes{ $NoteTypes{$KeyNoteType} } ) {
             delete $NoteTypes{$KeyNoteType};
         }
     }
-    $Param{NoteStrg} = $Self->{LayoutObject}->BuildSelection(
-        Data       => \%NoteTypes,
-        Name       => 'ArticleTypeID',
-        SelectedID => $Param{ArticleTypeID} || $Self->{Config}->{ArticleTypeDefault},
-    );
+
+    if ( $Param{ArticleTypeID} ) {
+        $Param{NoteStrg} = $Self->{LayoutObject}->BuildSelection(
+            Data       => \%NoteTypes,
+            Name       => 'ArticleTypeID',
+            SelectedID => $Param{ArticleTypeID},
+        );
+    }
+    else {
+        $Param{NoteStrg} = $Self->{LayoutObject}->BuildSelection(
+            Data          => \%NoteTypes,
+            Name          => 'ArticleTypeID',
+            SelectedValue => $Self->{Config}->{ArticleTypeDefault},
+        );
+    }
 
     # build next states string
     if ( $Self->{Config}->{State} ) {
@@ -566,6 +839,27 @@ sub _Mask {
         }
     }
 
+    # types
+    if ( $Self->{ConfigObject}->Get('Ticket::Type') && $Self->{Config}->{TicketType} ) {
+        my %Type = $Self->{TicketObject}->TicketTypeList(
+            %Param,
+            Action => $Self->{Action},
+            UserID => $Self->{UserID},
+        );
+        $Param{TypeStrg} = $Self->{LayoutObject}->BuildSelection(
+            Data         => \%Type,
+            PossibleNone => 1,
+            Name         => 'TypeID',
+            SelectedID   => $Param{TypeID},
+            Sort         => 'AlphanumericValue',
+            Translation  => 0,
+        );
+        $Self->{LayoutObject}->Block(
+            Name => 'Type',
+            Data => {%Param},
+        );
+    }
+
     # owner list
     if ( $Self->{Config}->{Owner} ) {
         my %AllGroupsMembers = $Self->{UserObject}->UserList( Type => 'Long', Valid => 1 );
@@ -574,7 +868,10 @@ sub _Mask {
         if ( !$Self->{ConfigObject}->Get('Ticket::ChangeOwnerToEveryone') ) {
             my %AllGroupsMembersNew;
             for my $TicketID ( @{ $Param{TicketIDs} } ) {
-                my %Ticket = $Self->{TicketObject}->TicketGet( TicketID => $TicketID );
+                my %Ticket = $Self->{TicketObject}->TicketGet(
+                    TicketID      => $TicketID,
+                    DynamicFields => 0,
+                );
                 my $GroupID = $Self->{QueueObject}->GetQueueGroupID( QueueID => $Ticket{QueueID} );
                 my %GroupMember = $Self->{GroupObject}->GroupMemberList(
                     GroupID => $GroupID,
@@ -590,10 +887,11 @@ sub _Mask {
             }
         }
         $Param{OwnerStrg} = $Self->{LayoutObject}->BuildSelection(
-            Data => { '' => '-', %AllGroupsMembers },
-            Name => 'OwnerID',
-            LanguageTranslation => 0,
-            SelectedID          => $Param{OwnerID},
+            Data         => \%AllGroupsMembers,
+            Name         => 'OwnerID',
+            Translation  => 0,
+            SelectedID   => $Param{OwnerID},
+            PossibleNone => 1,
         );
         $Self->{LayoutObject}->Block(
             Name => 'Owner',
@@ -601,7 +899,7 @@ sub _Mask {
         );
     }
 
-    # owner list
+    # responsible list
     if ( $Self->{ConfigObject}->Get('Ticket::Responsible') && $Self->{Config}->{Responsible} ) {
         my %AllGroupsMembers = $Self->{UserObject}->UserList( Type => 'Long', Valid => 1 );
 
@@ -609,7 +907,10 @@ sub _Mask {
         if ( !$Self->{ConfigObject}->Get('Ticket::ChangeOwnerToEveryone') ) {
             my %AllGroupsMembersNew;
             for my $TicketID ( @{ $Param{TicketIDs} } ) {
-                my %Ticket = $Self->{TicketObject}->TicketGet( TicketID => $TicketID );
+                my %Ticket = $Self->{TicketObject}->TicketGet(
+                    TicketID      => $TicketID,
+                    DynamicFields => 0,
+                );
                 my $GroupID = $Self->{QueueObject}->GetQueueGroupID( QueueID => $Ticket{QueueID} );
                 my %GroupMember = $Self->{GroupObject}->GroupMemberList(
                     GroupID => $GroupID,
@@ -627,8 +928,8 @@ sub _Mask {
         $Param{ResponsibleStrg} = $Self->{LayoutObject}->BuildSelection(
             Data => { '' => '-', %AllGroupsMembers },
             Name => 'ResponsibleID',
-            LanguageTranslation => 0,
-            SelectedID          => $Param{ResponsibleID},
+            Translation => 0,
+            SelectedID  => $Param{ResponsibleID},
         );
         $Self->{LayoutObject}->Block(
             Name => 'Responsible',
@@ -685,13 +986,43 @@ sub _Mask {
     if ( $Self->{ConfigObject}->Get('Ticket::Frontend::AccountTime') ) {
         $Param{TimeUnitsRequired} = (
             $Self->{ConfigObject}->Get('Ticket::Frontend::NeedAccountedTime')
-            ? 'Validate_Required'
+            ? 'Validate_DependingRequiredAND Validate_Depending_Subject'
             : ''
         );
+        $Param{TimeUnitsRequiredEmail} = (
+            $Self->{ConfigObject}->Get('Ticket::Frontend::NeedAccountedTime')
+            ? 'Validate_DependingRequiredAND Validate_Depending_EmailSubject'
+            : ''
+        );
+
+        if ( $Self->{ConfigObject}->Get('Ticket::Frontend::NeedAccountedTime') ) {
+            $Self->{LayoutObject}->Block(
+                Name => 'TimeUnitsLabelMandatory',
+                Data => { TimeUnitsRequired => $Param{TimeUnitsRequired} },
+            );
+            $Self->{LayoutObject}->Block(
+                Name => 'TimeUnitsLabelMandatoryEmail',
+                Data => { TimeUnitsRequired => $Param{TimeUnitsRequiredEmail} },
+            );
+        }
+        else {
+            $Self->{LayoutObject}->Block(
+                Name => 'TimeUnitsLabel',
+                Data => \%Param,
+            );
+            $Self->{LayoutObject}->Block(
+                Name => 'TimeUnitsLabelEmail',
+                Data => \%Param,
+            );
+        }
         $Self->{LayoutObject}->Block(
             Name => 'TimeUnits',
             Data => \%Param,
         );
+        $Self->{LayoutObject}->Block(
+            Name => 'TimeUnitsEmail',
+            Data => \%Param,
+        );
     }
 
     $Param{LinkTogetherYesNoOption} = $Self->{LayoutObject}->BuildSelection(
@@ -714,19 +1045,45 @@ sub _Mask {
         );
     }
 
-    # add rich text editor
+    # add rich text editor for note & email
     if ( $Self->{LayoutObject}->{BrowserRichText} ) {
+
+        # use height/width defined for this screen
+        $Param{RichTextHeight} = $Self->{Config}->{RichTextHeight} || 0;
+        $Param{RichTextWidth}  = $Self->{Config}->{RichTextWidth}  || 0;
+
         $Self->{LayoutObject}->Block(
             Name => 'RichText',
             Data => \%Param,
         );
+        $Self->{LayoutObject}->Block(
+            Name => 'RichTextEmail',
+            Data => \%Param,
+        );
     }
 
     # reload parent window
     if ( $Param{TicketsWereLocked} ) {
         $Self->{LayoutObject}->Block(
             Name => 'ParentReload',
+            Data => {
+                URL => $Self->{LastScreenOverview},
+                }
         );
+
+        # show undo link
+        $Self->{LayoutObject}->Block(
+            Name => 'UndoClosePopup',
+            Data => { %Param, TicketID => $Param{"LockedTickets"} },
+        );
+    }
+    else {
+        # show back link
+        $Self->{LayoutObject}->Block(
+            Name => 'CancelClosePopup',
+            Data => %Param
+        );
+
     }
 
     # get output back
