@@ -1,9 +1,10 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Kernel/System/MailAccount/POP3TLS.pm - lib for pop3/tls accounts
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::MailAccount::POP3TLS;
@@ -11,22 +12,9 @@ package Kernel::System::MailAccount::POP3TLS;
 use strict;
 use warnings;
 
-use Net::POP3;
+use Net::SSLGlue::POP3;
 
-use parent qw(Kernel::System::MailAccount::POP3);
-
-our @ObjectDependencies = (
-    'Kernel::System::Log',
-);
-
-# Use Net::SSLGlue::POP3 on systems with older Net::POP3 modules that cannot handle POP3TLS.
-BEGIN {
-    if ( !defined &Net::POP3::starttls ) {
-        ## nofilter(TidyAll::Plugin::OTRS::Perl::Require)
-        ## nofilter(TidyAll::Plugin::OTRS::Perl::SyntaxCheck)
-        require Net::SSLGlue::POP3;
-    }
-}
+use base qw(Kernel::System::MailAccount::POP3);
 
 sub Connect {
     my ( $Self, %Param ) = @_;
@@ -34,10 +22,8 @@ sub Connect {
     # check needed stuff
     for (qw(Login Password Host Timeout Debug)) {
         if ( !defined $Param{$_} ) {
-            return (
-                Successful => 0,
-                Message    => "Need $_!",
-            );
+            $Self->{LogObject}->Log( Priority => 'error', Message => "Need $_!" );
+            return;
         }
     }
 
@@ -51,10 +37,7 @@ sub Connect {
     );
 
     if ( !$PopObject ) {
-        return (
-            Successful => 0,
-            Message    => "$Type: Can't connect to $Param{Host}"
-        );
+        return ( Successful => 0, Message => "$Type: Can't connect to $Param{Host}" );
     }
 
     $PopObject->starttls(
