@@ -1,9 +1,10 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Kernel/System/PID.pm - all system pid functions
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::PID;
@@ -11,27 +12,55 @@ package Kernel::System::PID;
 use strict;
 use warnings;
 
-our @ObjectDependencies = (
-    'Kernel::Config',
-    'Kernel::System::DB',
-    'Kernel::System::Log',
-);
-
 =head1 NAME
 
 Kernel::System::PID - to manage PIDs
 
-=head1 DESCRIPTION
+=head1 SYNOPSIS
 
 All functions to manage process ids
 
 =head1 PUBLIC INTERFACE
 
-=head2 new()
+=over 4
 
-Don't use the constructor directly, use the ObjectManager instead:
+=cut
 
-    my $PIDObject = $Kernel::OM->Get('Kernel::System::PID');
+=item new()
+
+create an object
+
+    use Kernel::Config;
+    use Kernel::System::Encode;
+    use Kernel::System::Log;
+    use Kernel::System::Main;
+    use Kernel::System::DB;
+    use Kernel::System::PID;
+
+    my $ConfigObject = Kernel::Config->new();
+    my $EncodeObject = Kernel::System::Encode->new(
+        ConfigObject => $ConfigObject,
+    );
+    my $LogObject = Kernel::System::Log->new(
+        ConfigObject => $ConfigObject,
+        EncodeObject => $EncodeObject,
+    );
+    my $MainObject = Kernel::System::Main->new(
+        ConfigObject => $ConfigObject,
+        EncodeObject => $EncodeObject,
+        LogObject    => $LogObject,
+    );
+    my $DBObject = Kernel::System::DB->new(
+        ConfigObject => $ConfigObject,
+        EncodeObject => $EncodeObject,
+        LogObject    => $LogObject,
+        MainObject   => $MainObject,
+    );
+    my $PIDObject = Kernel::System::PID->new(
+        LogObject    => $LogObject,
+        ConfigObject => $ConfigObject,
+        DBObject     => $DBObject,
+    );
 
 =cut
 
@@ -42,13 +71,18 @@ sub new {
     my $Self = {};
     bless( $Self, $Type );
 
-    # get fqdn
-    $Self->{Host} = $Kernel::OM->Get('Kernel::Config')->Get('FQDN');
+    # check needed objects
+    for (qw(DBObject ConfigObject LogObject)) {
+        $Self->{$_} = $Param{$_} || die "Got no $_!";
+    }
+
+    # get common config options
+    $Self->{Host} = $Self->{ConfigObject}->Get('FQDN');
 
     return $Self;
 }
 
-=head2 PIDCreate()
+=item PIDCreate()
 
 create a new process id lock
 
@@ -78,10 +112,7 @@ sub PIDCreate {
 
     # check needed stuff
     if ( !$Param{Name} ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => 'Need Name'
-        );
+        $Self->{LogObject}->Log( Priority => 'error', Message => 'Need Name' );
         return;
     }
 
@@ -92,7 +123,7 @@ sub PIDCreate {
 
         my $TTL = $Param{TTL} || 3600;
         if ( $ProcessID{Created} > ( time() - $TTL ) ) {
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
+            $Self->{LogObject}->Log(
                 Priority => 'notice',
                 Message  => "Can't create PID $ProcessID{Name}, because it's already running "
                     . "($ProcessID{Host}/$ProcessID{PID})!",
@@ -100,10 +131,10 @@ sub PIDCreate {
             return;
         }
 
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
+        $Self->{LogObject}->Log(
             Priority => 'notice',
-            Message =>
-                "Removed PID ($ProcessID{Name}/$ProcessID{Host}/$ProcessID{PID}, because the TTL ($TTL sec) was exceeded!",
+            Message  => "Removed PID ($ProcessID{Name}/$ProcessID{Host}/$ProcessID{PID}, "
+                . "because 1 hour old!",
         );
     }
 
@@ -116,7 +147,7 @@ sub PIDCreate {
 
     # add new entry
     my $Time = time();
-    return if !$Kernel::OM->Get('Kernel::System::DB')->Do(
+    return if !$Self->{DBObject}->Do(
         SQL => '
             INSERT INTO process_id
             (process_name, process_id, process_host, process_create, process_change)
@@ -127,7 +158,7 @@ sub PIDCreate {
     return 1;
 }
 
-=head2 PIDGet()
+=item PIDGet()
 
 get process id lock info
 
@@ -142,18 +173,12 @@ sub PIDGet {
 
     # check needed stuff
     if ( !$Param{Name} ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => 'Need Name'
-        );
+        $Self->{LogObject}->Log( Priority => 'error', Message => 'Need Name' );
         return;
     }
 
-    # get database object
-    my $DBObject = $Kernel::OM->Get('Kernel::System::DB');
-
     # sql
-    return if !$DBObject->Prepare(
+    return if !$Self->{DBObject}->Prepare(
         SQL => '
             SELECT process_name, process_id, process_host, process_create, process_change
             FROM process_id
@@ -164,7 +189,7 @@ sub PIDGet {
 
     # fetch the result
     my %Data;
-    while ( my @Row = $DBObject->FetchrowArray() ) {
+    while ( my @Row = $Self->{DBObject}->FetchrowArray() ) {
         %Data = (
             PID     => $Row[1],
             Name    => $Row[0],
@@ -177,7 +202,7 @@ sub PIDGet {
     return %Data;
 }
 
-=head2 PIDDelete()
+=item PIDDelete()
 
 delete the process id lock
 
@@ -198,10 +223,7 @@ sub PIDDelete {
 
     # check needed stuff
     if ( !$Param{Name} ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => 'Need Name'
-        );
+        $Self->{LogObject}->Log( Priority => 'error', Message => 'Need Name' );
         return;
     }
 
@@ -217,11 +239,11 @@ sub PIDDelete {
         $SQL .= '
         AND process_host = ?';
 
-        push @Bind, \$Self->{Host};
+        push @Bind, \$Self->{Host}
     }
 
     # sql
-    return if !$Kernel::OM->Get('Kernel::System::DB')->Do(
+    return if !$Self->{DBObject}->Do(
         SQL  => $SQL,
         Bind => \@Bind,
     );
@@ -229,7 +251,7 @@ sub PIDDelete {
     return 1;
 }
 
-=head2 PIDUpdate()
+=item PIDUpdate()
 
 update the process id change time.
 this might be useful as a keep alive signal.
@@ -245,26 +267,20 @@ sub PIDUpdate {
 
     # check needed stuff
     if ( !$Param{Name} ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => 'Need Name'
-        );
+        $Self->{LogObject}->Log( Priority => 'error', Message => 'Need Name' );
         return;
     }
 
     my %PID = $Self->PIDGet( Name => $Param{Name} );
 
     if ( !%PID ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => 'Can not get PID'
-        );
+        $Self->{LogObject}->Log( Priority => 'error', Message => 'Can not get PID' );
         return;
     }
 
     # sql
     my $Time = time();
-    return if !$Kernel::OM->Get('Kernel::System::DB')->Do(
+    return if !$Self->{DBObject}->Do(
         SQL => '
             UPDATE process_id
             SET process_change = ?
@@ -277,12 +293,14 @@ sub PIDUpdate {
 
 1;
 
+=back
+
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<https://otrs.org/>).
+This software is part of the OTRS project (L<http://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (GPL). If you
-did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
 
 =cut
