@@ -1,6 +1,6 @@
 # --
 # Kernel/System/Stats/Dynamic/TicketSolutionResponseTime.pm - stats about ticket solution and response time
-# Copyright (C) 2001-2013 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -44,8 +44,8 @@ sub new {
     $Self->{ServiceObject}      = Kernel::System::Service->new( %{$Self} );
     $Self->{SLAObject}          = Kernel::System::SLA->new( %{$Self} );
     $Self->{TypeObject}         = Kernel::System::Type->new( %{$Self} );
-    $Self->{DynamicFieldObject} = Kernel::System::DynamicField->new(%Param);
-    $Self->{BackendObject}      = Kernel::System::DynamicField::Backend->new(%Param);
+    $Self->{DynamicFieldObject} = Kernel::System::DynamicField->new( %{$Self} );
+    $Self->{BackendObject}      = Kernel::System::DynamicField::Backend->new( %{$Self} );
 
     # get the dynamic fields for ticket object
     $Self->{DynamicField} = $Self->{DynamicFieldObject}->DynamicFieldListGet(
@@ -62,13 +62,32 @@ sub GetObjectName {
     return 'TicketSolutionResponseTime';
 }
 
+sub GetObjectBehaviours {
+    my ( $Self, %Param ) = @_;
+
+    my %Behaviours = (
+        ProvidesDashboardWidget => 0,
+    );
+
+    return %Behaviours;
+}
+
 sub GetObjectAttributes {
     my ( $Self, %Param ) = @_;
 
+    my $ValidAgent = 0;
+    if (
+        defined $Self->{ConfigObject}->Get('Stats::UseInvalidAgentInStats')
+        && ( $Self->{ConfigObject}->Get('Stats::UseInvalidAgentInStats') == 0 )
+        )
+    {
+        $ValidAgent = 1;
+    }
+
     # get user list
     my %UserList = $Self->{UserObject}->UserList(
         Type  => 'Long',
-        Valid => 0,
+        Valid => $ValidAgent,
     );
 
     # get state list
@@ -115,6 +134,7 @@ sub GetObjectAttributes {
             Element          => 'QueueIDs',
             Block            => 'MultiSelectField',
             Translation      => 0,
+            TreeView         => 1,
             Values           => \%QueueList,
         },
         {
@@ -152,6 +172,7 @@ sub GetObjectAttributes {
             Element          => 'CreatedQueueIDs',
             Block            => 'MultiSelectField',
             Translation      => 0,
+            TreeView         => 1,
             Values           => \%QueueList,
         },
         {
@@ -338,6 +359,7 @@ sub GetObjectAttributes {
                 Element          => 'ServiceIDs',
                 Block            => 'MultiSelectField',
                 Translation      => 0,
+                TreeView         => 1,
                 Values           => \%Service,
             },
             {
@@ -376,6 +398,26 @@ sub GetObjectAttributes {
         unshift @ObjectAttributes, \%ObjectAttribute1;
     }
 
+    if ( $Self->{ConfigObject}->Get('Ticket::ArchiveSystem') ) {
+
+        my %ObjectAttribute = (
+            Name             => 'Archive Search',
+            UseAsXvalue      => 0,
+            UseAsValueSeries => 0,
+            UseAsRestriction => 1,
+            Element          => 'SearchInArchive',
+            Block            => 'SelectField',
+            Translation      => 1,
+            Values           => {
+                ArchivedTickets    => 'Archived tickets',
+                NotArchivedTickets => 'Unarchived tickets',
+                AllTickets         => 'All tickets',
+            },
+        );
+
+        push @ObjectAttributes, \%ObjectAttribute;
+    }
+
     if ( $Self->{ConfigObject}->Get('Stats::UseAgentElementInStats') ) {
 
         my @ObjectAttributeAdd = (
@@ -461,27 +503,48 @@ sub GetObjectAttributes {
     for my $DynamicFieldConfig ( @{ $Self->{DynamicField} } ) {
         next DYNAMICFIELD if !IsHashRefWithData($DynamicFieldConfig);
 
+        # skip all fields not designed to be supported by statistics
+        my $IsStatsCondition = $Self->{BackendObject}->HasBehavior(
+            DynamicFieldConfig => $DynamicFieldConfig,
+            Behavior           => 'IsStatsCondition',
+        );
+
+        next DYNAMICFIELD if !$IsStatsCondition;
+
         my $PossibleValuesFilter;
 
-        # convert possible values key => value to key => key for ACLs usign a Hash slice
-        my %AclData = %{ $DynamicFieldConfig->{Config}->{PossibleValues} || {} };
-        @AclData{ keys %AclData } = keys %AclData;
-
-        # set possible values filter from ACLs
-        my $ACL = $Self->{TicketObject}->TicketAcl(
-            Action        => 'AgentStats',
-            Type          => 'DynamicField_' . $DynamicFieldConfig->{Name},
-            ReturnType    => 'Ticket',
-            ReturnSubType => 'DynamicField_' . $DynamicFieldConfig->{Name},
-            Data          => \%AclData || {},
-            UserID        => 1,
+        my $IsACLReducible = $Self->{BackendObject}->HasBehavior(
+            DynamicFieldConfig => $DynamicFieldConfig,
+            Behavior           => 'IsACLReducible',
         );
-        if ($ACL) {
-            my %Filter = $Self->{TicketObject}->TicketAclData();
 
-            # convert Filer key => key back to key => value using map
-            %{$PossibleValuesFilter}
-                = map { $_ => $DynamicFieldConfig->{Config}->{PossibleValues}->{$_} } keys %Filter;
+        if ($IsACLReducible) {
+
+            # get PossibleValues
+            my $PossibleValues = $Self->{BackendObject}->PossibleValuesGet(
+                DynamicFieldConfig => $DynamicFieldConfig,
+            );
+
+            # convert possible values key => value to key => key for ACLs using a Hash slice
+            my %AclData = %{ $PossibleValues || {} };
+            @AclData{ keys %AclData } = keys %AclData;
+
+            # set possible values filter from ACLs
+            my $ACL = $Self->{TicketObject}->TicketAcl(
+                Action        => 'AgentStats',
+                Type          => 'DynamicField_' . $DynamicFieldConfig->{Name},
+                ReturnType    => 'Ticket',
+                ReturnSubType => 'DynamicField_' . $DynamicFieldConfig->{Name},
+                Data          => \%AclData || {},
+                UserID        => 1,
+            );
+            if ($ACL) {
+                my %Filter = $Self->{TicketObject}->TicketAclData();
+
+                # convert Filer key => key back to key => value using map
+                %{$PossibleValuesFilter}
+                    = map { $_ => $PossibleValues->{$_} } keys %Filter;
+            }
         }
 
         # get field html
@@ -502,6 +565,8 @@ sub GetObjectAttributes {
                     Block            => 'MultiSelectField',
                     Values           => $DynamicFieldStatsParameter->{Values},
                     Translation      => 0,
+                    IsDynamicField   => 1,
+                    ShowAsTree       => $DynamicFieldConfig->{Config}->{TreeView} || 0,
                 );
                 push @ObjectAttributes, \%ObjectAttribute;
             }
@@ -527,6 +592,7 @@ sub GetObjectAttributes {
 sub GetStatTable {
     my ( $Self, %Param ) = @_;
     my @StatArray;
+
     if ( $Param{XValue}{Element} && $Param{XValue}{Element} eq 'KindsOfReporting' ) {
 
         for my $Row ( sort keys %{ $Param{TableStructure} } ) {
@@ -817,9 +883,17 @@ sub _ReportingValues {
                 # without the 'DynamicField_' prefix
                 next DYNAMICFIELD if $DynamicFieldConfig->{Name} ne $1;
 
+                # skip all fields not designed to be supported by statistics
+                my $IsStatsCondition = $Self->{BackendObject}->HasBehavior(
+                    DynamicFieldConfig => $DynamicFieldConfig,
+                    Behavior           => 'IsStatsCondition',
+                );
+
+                next DYNAMICFIELD if !$IsStatsCondition;
+
                 # get new search parameter
                 my $DynamicFieldStatsSearchParameter
-                    = $Self->{BackendObject}->CommonSearchFieldParameterBuild(
+                    = $Self->{BackendObject}->StatsSearchFieldParameterBuild(
                     DynamicFieldConfig => $DynamicFieldConfig,
                     Value              => $TicketSearch{$ParameterName},
                     );
@@ -830,6 +904,19 @@ sub _ReportingValues {
         }
     }
 
+    if ( $Self->{ConfigObject}->Get('Ticket::ArchiveSystem') ) {
+        $SearchAttributes->{SearchInArchive} ||= '';
+        if ( $SearchAttributes->{SearchInArchive} eq 'AllTickets' ) {
+            $TicketSearch{ArchiveFlags} = [ 'y', 'n' ];
+        }
+        elsif ( $SearchAttributes->{SearchInArchive} eq 'ArchivedTickets' ) {
+            $TicketSearch{ArchiveFlags} = ['y'];
+        }
+        else {
+            $TicketSearch{ArchiveFlags} = ['n'];
+        }
+    }
+
     # get the involved tickets
     my @TicketIDs = $Self->{TicketObject}->TicketSearch(
         UserID     => 1,
